            <dt{% if not property.has_details %} id="{{ property.id }}"{% endif %}>
              <a href="#{{ property.id }}" class="m-doc{% if not property.has_details %}-self{% endif %}">{{ property.name }}</a>{% if property.type_link %}: {{ property.type_link }}{% endif %} <span class="m-label m-flat {% if property.is_gettable and property.is_settable %}m-success{% elif property.is_gettable %}m-warning{% else %}m-danger{% endif %}">{% if property.is_gettable and property.is_settable %}get set{% elif property.is_gettable %}get{% else %}set{% endif %}{% if property.is_deletable %} del{% endif %}</span>
            </dt>
            <dd>{{ property.summary }}</dd>
