/* Generated using `./postprocess.py m-light.css m-theme-light.css m-documentation.css --no-import -o m-light.documentation.compiled.css`. Do not edit. */

/*
    This file is part of m.css.

    Copyright © 2017, 2018, 2019, 2020, 2021, 2022
              Vladimír Vondruš <mosra@centrum.cz>

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included
    in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
*/

a.m-doc, a.m-doc-self, a.m-doc-external,
ul.m-doc li.m-doc-expansible > a:first-child, ul.m-doc li.m-doc-collapsible > a:first-child,
.m-code.m-inverted.m-doc-include > a {
  text-decoration: none;
}
a.m-doc, a.m-doc-self {
  font-weight: bold;
}
.m-thin a.m-doc, .m-thin a.m-doc-self {
  font-weight: normal;
}
ul.m-doc li.m-doc-expansible > a:first-child,
ul.m-doc li.m-doc-collapsible > a:first-child,
ul.m-doc li.m-doc-expansible > a:first-child:hover,
ul.m-doc li.m-doc-expansible > a:first-child:focus,
ul.m-doc li.m-doc-expansible > a:first-child:active,
ul.m-doc li.m-doc-collapsible > a:first-child:hover,
ul.m-doc li.m-doc-collapsible > a:first-child:focus,
ul.m-doc li.m-doc-collapsible > a:first-child:active {
  color: #000000;
}
a.m-doc-self,
ul.m-doc li.m-doc-expansible > a:first-child::before,
ul.m-doc li.m-doc-collapsible > a:first-child::before {
  color: #cb4b16;
}
a.m-doc-self:hover, a.m-doc-self:focus, a.m-doc-self:active,
ul.m-doc li.m-doc-expansible > a:first-child:hover::before,
ul.m-doc li.m-doc-expansible > a:first-child:focus::before,
ul.m-doc li.m-doc-expansible > a:first-child:active::before,
ul.m-doc li.m-doc-collapsible > a:first-child:hover::before,
ul.m-doc li.m-doc-collapsible > a:first-child:focus::before,
ul.m-doc li.m-doc-collapsible > a:first-child:active::before {
  color: #802f0e;
}
h3 a.m-doc-external {
  font-weight: normal;
}
span.m-doc-wrap-bumper {
  margin-right: -1rem;
}
span.m-doc-wrap {
  margin-left: 1rem;
  display: inline-block;
  vertical-align: text-top;
  white-space: pre-line;
  max-width: 100%;
}
dl.m-doc dd {
  margin-bottom: 0.5rem;
}
dl.m-doc dd {
  margin-left: 0;
  padding-left: 2.5rem;
}
dl.m-doc dt:target, dl.m-doc dt:target + dd {
  margin-left: -1.0rem;
  border-left-style: solid;
  border-left-width: 0.25rem;
  border-color: #cb4b16;
}
dl.m-doc dt:target { padding-left: 0.75rem; }
dl.m-doc dt:target + dd { padding-left: 3.25rem; }
ul.m-doc {
  list-style: none;
  margin-left:  1.0375rem;
  padding-left: 0.9rem;
  border-left-color: #f7e3db;
  border-left-width: 0.0625rem;
  border-left-style: solid;
}
ul.m-doc li {
  text-indent: -1rem;
  padding-left: 1rem;
}
ul.m-doc li.m-doc-expansible > ul {
  display: none;
}
ul.m-doc li.m-doc-expansible, ul.m-doc li.m-doc-collapsible {
  padding-left: 0.6rem;
}
ul.m-doc li.m-doc-expansible > ul.m-doc, ul.m-doc li.m-doc-collapsible > ul.m-doc {
  margin-left: 0.5rem;
}
ul.m-doc li.m-doc-expansible > a:first-child::before, ul.m-doc li.m-doc-collapsible > a:first-child::before {
  background-color: #ffffff;
  display: inline-block;
  width: 0.4rem;
  font-weight: bold;
}
ul.m-doc li.m-doc-expansible > a:first-child::before { content: '⊕'; }
ul.m-doc li.m-doc-collapsible > a:first-child::before { content: '⊖'; }
h1 .m-doc-template, h1 .m-doc-include {
  font-size: 1.3rem;
  font-weight: normal;
}
h1 .m-doc-include:last-child {
  margin-bottom: -0.5rem;
}
h3 .m-doc-template, h3 .m-doc-include {
  font-size: 1rem;
  font-weight: normal;
}
.m-doc-template, dl.m-doc dd, ul.m-doc li > span.m-doc {
  color: #bdbdbd;
}
dl.m-doc dd svg.m-math, ul.m-doc li > span.m-doc svg.m-math {
  fill: #bdbdbd;
}
.m-doc-template a, dl.m-doc dd a, ul.m-doc li > span.m-doc a {
  color: #c0c0c0;
}
.m-doc-template a:hover, .m-doc-template a:focus, .m-doc-template a:active,
dl.m-doc dd a:hover, dl.m-doc dd a:focus, dl.m-doc dd a:active,
ul.m-doc li > span.m-doc a:hover, ul.m-doc li > span.m-doc a:focus, ul.m-doc li > span.m-doc a:active {
  color: #949494;
}
.m-code.m-inverted.m-doc-include > a:link,
.m-code.m-inverted.m-doc-include > a:visited {
  opacity: 0.6666;
}
.m-code.m-inverted.m-doc-include > a:hover,
.m-code.m-inverted.m-doc-include > a:focus,
.m-code.m-inverted.m-doc-include > a:active {
  opacity: 1;
}
article section.m-doc-details > div {
  margin-top: 0;
  margin-left: 0;
  margin-right: 0;
  position: relative;
  padding: 1rem;
}
article section.m-doc-details > div::before {
  position: absolute;
  content: ' ';
  top: 0;
  bottom: 0;
  left: 0;
  right: 0;
  z-index: -1;
  border-style: solid;
  border-width: 0.125rem;
  border-radius: 0.2rem;
  border-color: #fbf0ec;
}
article section.m-doc-details > div > h3:first-child {
  position: relative;
  margin: -1rem -1rem 1rem -1rem;
  padding: 0.5rem 1rem;
  background-color: #fbf0ec;
  border-top-left-radius: 0.2rem;
  border-top-right-radius: 0.2rem;
  border-bottom-left-radius: 0;
  border-bottom-right-radius: 0;
}
article section.m-doc-details:target {
  border-color: transparent;
}
article section.m-doc-details:target > div {
  z-index: 1;
}
.m-container-inflatable > .m-row > [class*='m-col-'] section.m-doc-details > div {
  margin-left: -1rem;
  margin-right: -1rem;
}
.m-container-inflatable section.m-doc-details:target > div > h3:first-child,
.m-container-inflatable section.m-doc-details:target section > div > h3:first-child {
  margin-left: -1.0rem;
  border-left-style: solid;
  border-left-color: #802f0e;
  border-left-width: 0.25rem;
  padding-left: 0.75rem;
}
.m-container-inflatable section.m-doc-details:target > div::before,
.m-container-inflatable section-dox-details:target section > div.m::before {
  border-left-width: 0.25rem;
  border-left-color: #cb4b16;
}
a.m-doc-search-icon {
  padding-left: 1rem;
  padding-right: 1rem;
}
a.m-doc-search-icon svg {
  fill: #000000;
}
body > header > nav #m-navbar-collapse a.m-doc-search-icon svg {
  vertical-align: -5%;
}
a.m-doc-search-icon:focus svg, a.m-doc-search-icon:hover svg, a.m-doc-search-icon:active svg {
  fill: #cb4b16;
}
.m-doc-search {
  display: none;
  z-index: 10;
  position: fixed;
  left: 0;
  right: 0;
  top: 0;
  bottom: 0;
  background-color: rgba(255, 255, 255, 0.75);
}
.m-doc-search:target {
  display: block;
}
.m-doc-search > a {
  display: block;
  position: absolute;
  left: 0;
  right: 0;
  top: 0;
  bottom: 0;
}
.m-doc-search-header {
  margin-top: 2.5rem;
  padding: 0.5rem 1rem;
  height: 2rem;
}
.m-doc-search-header > div:first-child {
  float: right;
}
.m-doc-search-content {
  background-color: #ffffff;
  border-radius: 0.2rem;
  padding: 1rem;
}
.m-doc-search input {
  width: 100%;
  height: 3rem;
  font-size: 1.2rem;
  border-width: 0;
  color: #000000;
  background-color: #fbf0ec;
  border-radius: 0.2rem;
  margin-bottom: 1rem;
  padding: 0 1rem;
}
.m-doc-search #search-notfound {
  display: none;
}
.m-doc-search ul#search-results {
  list-style-type: none;
  padding-left: 0;
  max-height: calc(100vh - 12.5rem);
  overflow-y: auto;
  display: none;
}
.m-doc-search ul#search-results li a {
  display: block;
  padding-left: 1rem;
  padding-right: 1rem;
  text-decoration: none;
  width: 100%;
  line-height: 1.5rem;
  color: #000000;
}
.m-doc-search ul#search-results li a > div {
  white-space: nowrap;
  overflow: hidden;
}
.m-doc-search ul#search-results li a > div:not(.m-doc-search-alias) {
  direction: rtl;
}
.m-doc-search ul#search-results li a .m-label {
  float: right;
  line-height: 1rem;
  margin-top: 0.1rem;
  margin-left: 0.25rem;
}
.m-doc-search ul#search-results li a .m-label.m-flat {
  margin-right: -0.75rem;
}
.m-doc-search ul#search-results li#search-current a {
  background-color: #fbf0ec;
}
.m-doc-search ul#search-results li#search-current.m-doc-search-copied a {
  background-color: #4dd376;
}
.m-doc-search-typed {
  color: #ea7944;
}
.m-doc-search input[type="search"] { -webkit-appearance: textfield; }
.m-doc-search input[type="search"]::-webkit-search-decoration,
.m-doc-search input[type="search"]::-webkit-search-cancel-button,
.m-doc-search input[type="search"]::-webkit-search-results-button,
.m-doc-search input[type="search"]::-webkit-search-results-decoration {
  display: none;
}
